@@ -1,4 +1,4 @@
-""" SAM macro definitions """
+﻿""" SAM macro definitions """
 from six import string_types
 
 import samtranslator.model.eventsources
@@ -53,7 +53,8 @@ class SamFunction(SamResourceMacro):
         'Layers': PropertyType(False, list_of(one_of(is_str(), is_type(dict)))),
 
         # Intrinsic functions in value of Alias property are not supported, yet
-        'AutoPublishAlias': PropertyType(False, one_of(is_str()))
+        'AutoPublishAlias': PropertyType(False, one_of(is_str())),
+        'VersionDescription': PropertyType(False, is_str())
     }
     event_resolver = ResourceTypeResolver(samtranslator.model.eventsources, samtranslator.model.eventsources.pull,
                                           samtranslator.model.eventsources.push,
@@ -264,8 +265,11 @@ class SamFunction(SamResourceMacro):
         event_resources = {}
         if self.Events:
             for logical_id, event_dict in self.Events.items():
-                event_source = self.event_resolver.resolve_resource_type(event_dict).from_dict(
-                    self.logical_id + logical_id, event_dict, logical_id)
+                try:
+                    event_source = self.event_resolver.resolve_resource_type(event_dict).from_dict(
+                        self.logical_id + logical_id, event_dict, logical_id)
+                except TypeError as e:
+                    raise InvalidEventException(logical_id, "{}".format(e))
                 event_resources[logical_id] = event_source.resources_to_link(resources)
         return event_resources
 
@@ -286,8 +290,11 @@ class SamFunction(SamResourceMacro):
         resources = []
         if self.Events:
             for logical_id, event_dict in self.Events.items():
-                eventsource = self.event_resolver.resolve_resource_type(event_dict).from_dict(
-                    lambda_function.logical_id + logical_id, event_dict, logical_id)
+                try:
+                    eventsource = self.event_resolver.resolve_resource_type(event_dict).from_dict(
+                        lambda_function.logical_id + logical_id, event_dict, logical_id)
+                except TypeError as e:
+                    raise InvalidEventException(logical_id, "{}".format(e))
 
                 kwargs = {
                     # When Alias is provided, connect all event sources to the alias and *not* the function
@@ -359,6 +366,7 @@ class SamFunction(SamResourceMacro):
 
         lambda_version = LambdaVersion(logical_id=logical_id, attributes=attributes)
         lambda_version.FunctionName = function.get_runtime_attr('name')
+        lambda_version.Description = self.VersionDescription
 
         return lambda_version
 
@@ -373,7 +381,7 @@ class SamFunction(SamResourceMacro):
         """
 
         if not name:
-            raise ValueError("Alias name is required to create an alias")
+            raise InvalidResourceException(self.logical_id, "Alias name is required to create an alias")
 
         logical_id = "{id}Alias{suffix}".format(id=function.logical_id, suffix=name)
         alias = LambdaAlias(logical_id=logical_id, attributes=self.get_passthrough_resource_attributes())
@@ -434,6 +442,7 @@ class SamApi(SamResourceMacro):
         'MinimumCompressionSize': PropertyType(False, is_type(int)),
         'Cors': PropertyType(False, one_of(is_str(), is_type(dict))),
         'Auth': PropertyType(False, is_type(dict)),
+        'GatewayResponses': PropertyType(False, is_type(dict)),
         'AccessLogSetting': PropertyType(False, is_type(dict)),
         'CanarySetting': PropertyType(False, is_type(dict)),
         'TracingEnabled': PropertyType(False, is_type(bool))
@@ -469,6 +478,7 @@ class SamApi(SamResourceMacro):
                                      minimum_compression_size=self.MinimumCompressionSize,
                                      cors=self.Cors,
                                      auth=self.Auth,
+                                     gateway_responses=self.GatewayResponses,
                                      access_log_setting=self.AccessLogSetting,
                                      canary_setting=self.CanarySetting,
                                      tracing_enabled=self.TracingEnabled,
