@@ -1,8 +1,9 @@
-import copy
+﻿import copy
 from six import string_types
 
 from samtranslator.model.intrinsics import ref
 from samtranslator.model.intrinsics import make_conditional
+from samtranslator.model.exceptions import InvalidDocumentException, InvalidTemplateException
 
 
 class SwaggerEditor(object):
@@ -16,6 +17,7 @@ class SwaggerEditor(object):
     _OPTIONS_METHOD = "options"
     _X_APIGW_INTEGRATION = 'x-amazon-apigateway-integration'
     _CONDITIONAL_IF = "Fn::If"
+    _X_APIGW_GATEWAY_RESPONSES = 'x-amazon-apigateway-gateway-responses'
     _X_ANY_METHOD = 'x-amazon-apigateway-any-method'
 
     def __init__(self, doc):
@@ -33,6 +35,7 @@ class SwaggerEditor(object):
         self._doc = copy.deepcopy(doc)
         self.paths = self._doc["paths"]
         self.security_definitions = self._doc.get("securityDefinitions", {})
+        self.gateway_responses = self._doc.get(self._X_APIGW_GATEWAY_RESPONSES, {})
 
     def get_path(self, path):
         path_dict = self.paths.get(path)
@@ -122,14 +125,17 @@ class SwaggerEditor(object):
 
         if not isinstance(path_dict, dict):
             # Either customers has provided us an invalid Swagger, or this class has messed it somehow
-            raise ValueError("Value of '{}' path must be a dictionary according to Swagger spec".format(path))
+            raise InvalidDocumentException(
+                [InvalidTemplateException("Value of '{}' path must be a dictionary according to Swagger spec."
+                                          .format(path))])
 
         if self._CONDITIONAL_IF in path_dict:
             path_dict = path_dict[self._CONDITIONAL_IF][1]
 
         path_dict.setdefault(method, {})
 
-    def add_lambda_integration(self, path, method, integration_uri, condition=None):
+    def add_lambda_integration(self, path, method, integration_uri,
+                               method_auth_config=None, api_auth_config=None, condition=None):
         """
         Adds aws_proxy APIGW integration to the given path+method.
 
@@ -156,6 +162,15 @@ class SwaggerEditor(object):
                 'uri': integration_uri
         }
 
+        method_auth_config = method_auth_config or {}
+        api_auth_config = api_auth_config or {}
+        if method_auth_config.get('Authorizer') == 'AWS_IAM' \
+           or api_auth_config.get('DefaultAuthorizer') == 'AWS_IAM' and not method_auth_config:
+            self.paths[path][method][self._X_APIGW_INTEGRATION]['credentials'] = self._generate_integration_credentials(
+                method_invoke_role=method_auth_config.get('InvokeRole'),
+                api_invoke_role=api_auth_config.get('InvokeRole')
+            )
+
         # If 'responses' key is *not* present, add it with an empty dict as value
         path_dict[method].setdefault('responses', {})
 
@@ -169,6 +184,13 @@ class SwaggerEditor(object):
         """
         self.paths[path] = make_conditional(condition, self.paths[path])
 
+    def _generate_integration_credentials(self, method_invoke_role=None, api_invoke_role=None):
+        return self._get_invoke_role(method_invoke_role or api_invoke_role)
+
+    def _get_invoke_role(self, invoke_role):
+        CALLER_CREDENTIALS_ARN = 'arn:aws:iam::*:user/*'
+        return invoke_role if invoke_role and invoke_role != 'CALLER_CREDENTIALS' else CALLER_CREDENTIALS_ARN
+
     def iter_on_path(self):
         """
         Yields all the paths available in the Swagger. As a caller, if you add new paths to Swagger while iterating,
@@ -369,8 +391,8 @@ class SwaggerEditor(object):
         """
         self.security_definitions = self.security_definitions or {}
 
-        for authorizerName, authorizer in authorizers.items():
-            self.security_definitions[authorizerName] = authorizer.generate_swagger()
+        for authorizer_name, authorizer in authorizers.items():
+            self.security_definitions[authorizer_name] = authorizer.generate_swagger()
 
     def set_path_default_authorizer(self, path, default_authorizer, authorizers):
         """
@@ -409,7 +431,6 @@ class SwaggerEditor(object):
     def set_method_authorizer(self, path, method_name, authorizer_name, authorizers, default_authorizer,
                               is_default=False):
         normalized_method_name = self._normalize_method_name(method_name)
-
         # It is possible that the method could have two definitions in a Fn::If block.
         for method_definition in self.get_method_contents(self.get_path(path)[normalized_method_name]):
 
@@ -418,7 +439,10 @@ class SwaggerEditor(object):
                 continue
             existing_security = method_definition.get('security', [])
             # TEST: [{'sigv4': []}, {'api_key': []}])
-            authorizer_names = set(authorizers.keys())
+            authorizer_list = ['AWS_IAM']
+            if authorizers:
+                authorizer_list.extend(authorizers.keys())
+            authorizer_names = set(authorizer_list)
             existing_non_authorizer_security = []
             existing_authorizer_security = []
 
@@ -473,6 +497,33 @@ class SwaggerEditor(object):
             if security:
                 method_definition['security'] = security
 
+                # The first element of the method_definition['security'] should be AWS_IAM
+                # because authorizer_list = ['AWS_IAM'] is hardcoded above
+                if 'AWS_IAM' in method_definition['security'][0]:
+                    aws_iam_security_definition = {
+                        'AWS_IAM': {
+                            'x-amazon-apigateway-authtype': 'awsSigv4',
+                            'type': 'apiKey',
+                            'name': 'Authorization',
+                            'in': 'header'
+                        }
+                    }
+                    if not self.security_definitions:
+                        self.security_definitions = aws_iam_security_definition
+                    elif 'AWS_IAM' not in self.security_definitions:
+                        self.security_definitions.update(aws_iam_security_definition)
+
+    def add_gateway_responses(self, gateway_responses):
+        """
+        Add Gateway Response definitions to Swagger.
+
+        :param dict gateway_responses: Dictionary of GatewayResponse configuration which gets translated.
+        """
+        self.gateway_responses = self.gateway_responses or {}
+
+        for response_type, response in gateway_responses.items():
+            self.gateway_responses[response_type] = response.generate_swagger()
+
     @property
     def swagger(self):
         """
@@ -486,6 +537,8 @@ class SwaggerEditor(object):
 
         if self.security_definitions:
             self._doc["securityDefinitions"] = self.security_definitions
+        if self.gateway_responses:
+            self._doc[self._X_APIGW_GATEWAY_RESPONSES] = self.gateway_responses
 
         return copy.deepcopy(self._doc)
 
